<!DOCTYPE html>
<html>
<head>
    <meta charset="UTF-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <title>QUANTEASE 基本面分析</title>
    <link rel="stylesheet" href="https://cdn.staticfile.org/twitter-bootstrap/3.3.7/css/bootstrap.min.css">  
    <script src="https://cdn.staticfile.org/jquery/2.1.1/jquery.min.js"></script>
    <script src="https://cdn.staticfile.org/twitter-bootstrap/3.3.7/js/bootstrap.min.js"></script>
</head>
<body>
	<div class="header">
		<p style="margin-bottom:20px;margin-top:0;margin-left: 0; color: white; font-family: Arial; background-color:black;text-align:center;height:25px">
		   <span style="width:20%;">QUANTEASE 让量化举重若轻
		   </span>
		</p>
	</div>
<div class="container">
	<div class="row clearfix">
		<div class="col-md-2 column">
			<div class="panel panel-info">
				<div class="panel-heading">
					<h3 class="panel-title">
						基本面分析
					</h3>
				</div>
				<div class="list-group">
                    <ul class="nav">
                        <li>
                            <a href="#" class="list-group-item active">综合</a>
                        </li>
                        <li>
                            <a href="#" class="list-group-item">仓单</a>
                        </li>
                        <li>
                            <a href="#" class="list-group-item">展期</a>
                        </li>
                        <li>
                            <a href="#" class="list-group-item">基差</a>
                        </li>
                        <li>
                            <a href="#" class="list-group-item">龙虎榜</a>
                        </li>
                        <li>
                            <a href="#" class="list-group-item">持仓</a>
                        </li>
                        <li>
                            <a href="#" class="list-group-item">月历效应</a>
                        </li>
                    </ul>
                
                </div>
			</div>
		</div>
		<div class="col-md-10 column">
			<div>
				<h3 id="content" style="text-align:center">综合</h3>
			</div>
			<table class="table">
				<thead>
					<tr>
						<th>
							编号
						</th>
						<th>
							产品
						</th>
						<th>
							交付时间
						</th>
						<th>
							状态
						</th>
					</tr>
				</thead>
				<tbody>
					<tr>
						<td>
							1
						</td>
						<td>
							TB - Monthly
						</td>
						<td>
							01/04/2012
						</td>
						<td>
							Default
						</td>
					</tr>
					<tr class="success">
						<td>
							1
						</td>
						<td>
							TB - Monthly
						</td>
						<td>
							01/04/2012
						</td>
						<td>
							Approved
						</td>
					</tr>
					<tr class="error">
						<td>
							2
						</td>
						<td>
							TB - Monthly
						</td>
						<td>
							02/04/2012
						</td>
						<td>
							Declined
						</td>
					</tr>
					<tr class="warning">
						<td>
							3
						</td>
						<td>
							TB - Monthly
						</td>
						<td>
							03/04/2012
						</td>
						<td>
							Pending
						</td>
					</tr>
					<tr class="info">
						<td>
							4
						</td>
						<td>
							TB - Monthly
						</td>
						<td>
							04/04/2012
						</td>
						<td>
							Call in to confirm
						</td>
					</tr>
				</tbody>
			</table>
		</div>
	</div>
</div>
<script>
    $(function() {
        $(".list-group > .nav > li > a").on("click", function(e) {
            //alert(e.currentTarget.textContent);
		 	var listItems = $(".list-group-item");
              
			  // Remove 'active' tag for all list items
			  for (let i = 0; i < listItems.length; i++) {
				  listItems[i].classList.remove("active");
			  }
				
			  // Add 'active' tag for currently selected item
			  this.classList.add("active");
              console.log(this.innerText)
			  $("#content").text(this.text);

        });
		 // Select all list items
        
    });

</script>
</body>